#!/usr/bin/python
import sys
import time
import numpy as np
import matplotlib
#matplotlib.use('Agg')
import matplotlib.pyplot as plt
import csv
import scipy.ndimage.filters
import scipy.interpolate
import six
if six.PY2:
    import ConfigParser as configparser
else:
    import configparser
    
cfg = configparser.ConfigParser(inline_comment_prefixes=(';','#',))
cfg.read('linkit.conf')
cfg.sections()

if (len(sys.argv) <= 1):
    fn = cfg.get('main', 'data_filename')       # Read data input filename from config file
else:
    fn = sys.argv[1]                               # Read data input filename from command line arg

chart_name   = cfg.get('plot', 'chart_title')
ref_range    = float(cfg.get('main', 'reference_value'))
baseline_col = int(cfg.get('main', 'baseline_ref_column'))
axis_y_max   = float(cfg.get('plot', 'y_max'))
axis_y_min   = float(cfg.get('plot', 'y_min'))
axis_y_step  = float(cfg.get('plot', 'y_resolution'))
axis_y_label = cfg.get('plot', 'y_label')
axis_x_max   = float(cfg.get('plot', 'x_max'))
axis_x_min   = float(cfg.get('plot', 'x_min'))
axis_x_step  = float(cfg.get('plot', 'x_resolution'))
axis_x_label = cfg.get('plot', 'x_label')

dataset_name1 = cfg.get('main', 'dataset_name1')
dataset_name2 = cfg.get('main', 'dataset_name2')
dataset_name3 = cfg.get('main', 'dataset_name3')
dataset_name4 = cfg.get('main', 'dataset_name4')
dataset_name5 = cfg.get('main', 'dataset_name5')
dataset_name6 = cfg.get('main', 'dataset_name6')
dataset_name7 = cfg.get('main', 'dataset_name7')
dataset_name8 = cfg.get('main', 'dataset_name8')

dataset_color1 = cfg.get('plot', 'dataset_color1')
dataset_color2 = cfg.get('plot', 'dataset_color2')
dataset_color3 = cfg.get('plot', 'dataset_color3')
dataset_color4 = cfg.get('plot', 'dataset_color4')
dataset_color5 = cfg.get('plot', 'dataset_color5')
dataset_color6 = cfg.get('plot', 'dataset_color6')
dataset_color7 = cfg.get('plot', 'dataset_color7')
dataset_color8 = cfg.get('plot', 'dataset_color8')
    
mfc_name = "DUT"
meter_name = "3458A"
real_row = 2

#if len(sys.argv) < 2:
#    print("Need an argument! <file_name> [<data_row_number>] [<dut_name>] [<meter_name>]")
#    quit()
#
#if len(sys.argv) >= 3:
#    real_row = int(sys.argv[2])
#
#if len(sys.argv) >= 4:
#    chart_name = sys.argv[3]
#
#if len(sys.argv) == 5:
#    meter_name = sys.argv[4]

ideal = []
ideal2 = []
ideal3 = []
real = []
real2 = []
real3 = []
real4 = []
real5 = []

print (baseline_col)

with open(fn) as csvfile:
    spamreader = csv.reader(csvfile, delimiter=';', quotechar='|')
    is_first = True
    for row in spamreader:
        if is_first:
            is_first = False
            continue
        #print ("%f" % float(row[1]))
        ideal.extend([float(row[baseline_col])])
        ideal2.extend([(float(row[2]) + float(row[3]) ) / 2])
        ideal3.extend([(float(row[2]) + float(row[3]) + float(row[4]) ) / 3])
        real.extend([float(row[2])])
        real2.extend([float(row[3])])
        real3.extend([float(row[4])])
        #real4.extend([float(row[5])])
        #real5.extend([float(row[1])])

#print (real2)

p = np.polyfit(real,ideal,1)
pv = np.polyval(p,real)
diff = ideal-pv
diff_ppm = (diff/ref_range)*1000000.0

p2 = np.polyfit(real2,ideal,1)
pv2 = np.polyval(p2,real2)
diff2 = ideal-pv2
diff_ppm2 = (diff2/ref_range)*1000000.0

p3 = np.polyfit(real3,ideal,1)
pv3 = np.polyval(p3,real3)
diff3 = ideal-pv3
diff_ppm3 = (diff3/ref_range)*1000000.0
#
p4 = np.polyfit(real3,ideal2,1)
pv4 = np.polyval(p4,real3)
diff4 = ideal2-pv4
diff_ppm4 = (diff4/ref_range)*1000000.0
#
p5 = np.polyfit(ideal,ideal3,1)
pv5 = np.polyval(p5,ideal)
diff5 = ideal3-pv5
diff_ppm5 = (diff5/ref_range)*1000000.0

#diff_ppm6 = diff_ppm - diff_ppm2 #(diff3/10.0)*1000000.0
#diff_ppm7 = diff_ppm5 - diff_ppm3 #(diff3/10.0)*1000000.0

inl_max = np.amax(diff_ppm)
inl_min = np.amin(diff_ppm)
inl_delta = inl_max - inl_min
inl_span = inl_delta / 2.0

#print(chart_name + " INL Max: %f, Min %f" % (inl_max , inl_min))
#print(chart_name + " INL +-%f" % inl_span)

fig, ax = plt.subplots()
idx = 1

ax.patch.set_facecolor('#eeeeff')
ax.set_clip_on(False)

w = float(cfg.get('main', 'plot_w')) / 100
h = float(cfg.get('main', 'plot_h')) / 100
ws = float(cfg.get('main', 'plot_w_s')) / 100
hs = float(cfg.get('main', 'plot_h_s')) / 100
fig.set_size_inches(w,h)
plt.tight_layout(rect=[0.02, 0.02, 1.0, 0.98], w_pad=0, h_pad=0.5)                                # Reduce margins around the graph

filtered1 = scipy.ndimage.filters.gaussian_filter1d(diff_ppm, sigma=1)
filtered2 = scipy.ndimage.filters.gaussian_filter1d(diff_ppm2, sigma=2)
filtered3 = scipy.ndimage.filters.gaussian_filter1d(diff_ppm3, sigma=2)
filtered4 = scipy.ndimage.filters.gaussian_filter1d(diff_ppm4, sigma=2)
filtered5 = scipy.ndimage.filters.gaussian_filter1d(diff_ppm5, sigma=2)
#filtered6 = scipy.ndimage.filters.gaussian_filter1d(diff_ppm6, sigma=2)
#filtered7 = scipy.ndimage.filters.gaussian_filter1d(diff_ppm7, sigma=2)

print (filtered1)
#for diff_ppm in diff_ppms:
#ax.plot(ideal, diff_ppm)
ax.plot(ideal, filtered1, dataset_color1, label='Data %i' % (0), linewidth=2.0)
ax.plot(ideal, filtered2, dataset_color2, label='Data %i' % (1), linewidth=2.0)
ax.plot(ideal, filtered3, dataset_color3, label='Data %i' % (2), linewidth=2.0)
ax.plot(ideal, filtered4, dataset_color4, label='Data %i' % (3), linewidth=4.0)
ax.plot(ideal, filtered5, '#888888', label='Data %i' % (4), marker='.', linewidth=1.0)
#ax.plot(ideal, filtered6, colors[5], label='Data %i' % (6), linewidth=2.0)
#ax.plot(ideal, filtered7, colors[6], label='Data %i' % (7), linewidth=2.0)
#    idx = idx + 1
#plt.text(-0.105, 1.01, "xDevs.com", fontsize=15)
#plt.text(-0.105, -1.11, "https://github.com/tin-/linkit", fontsize=13)

ax.axis([axis_x_min, axis_x_max, axis_y_min, axis_y_max])

ax.legend([dataset_name1, dataset_name2, dataset_name3, dataset_name4, dataset_name5, dataset_name6, dataset_name7, dataset_name8])

plt.text(0.08, -0.08,'xDevs.com | https://github.com/tin-/linkit',
     horizontalalignment='center',
     verticalalignment='center',
     transform = ax.transAxes)

start, end = ax.get_ylim()
ax.yaxis.set_ticks(np.arange(start, end, axis_y_step))
startx, endx = ax.get_xlim()
ax.xaxis.set_ticks(np.arange(startx, endx, axis_x_step))

ax.set(xlabel=axis_x_label, ylabel=axis_y_label,
       title='%s' % (chart_name))
ax.grid()

fig.savefig(cfg.get('main', 'plot_out_filename'), facecolor=fig.get_facecolor(), transparent=False)
ax.yaxis.set_ticks(np.arange(start, end, axis_y_step * 2))
plt.tight_layout(rect=[0.025, 0.025, 1.0, 0.98], w_pad=0, h_pad=0.5)                                # Reduce margins around the graph
figs = fig
figs.set_size_inches(ws,hs)
figs.savefig(cfg.get('main', 'plot_out_filename_small'), facecolor=figs.get_facecolor(), transparent=False)
plt.show()

